@@ -20,12 +20,21 @@ const CACHE_XATTR: &str = "user.emupart";
 
 #[derive(Debug, Default, Serialize, Deserialize)]
 pub struct GameDb {
-    pub description: String,
-    pub date: Option<String>,
-    pub games: HashMap<String, Game>,
+    description: String,
+    games: HashMap<String, Game>,
 }
 
 impl GameDb {
+    #[inline]
+    pub fn new(description: String, games: HashMap<String, Game>) -> Self {
+        Self { description, games }
+    }
+
+    #[inline]
+    pub fn description(&self) -> &str {
+        self.description.as_str()
+    }
+
     #[inline]
     pub fn is_game(&self, game: &str) -> bool {
         self.games.contains_key(game)
@@ -37,7 +46,7 @@ impl GameDb {
     }
 
     #[inline]
-    pub fn games_iter(&self) -> impl ExactSizeIterator<Item = &Game> {
+    pub fn games_iter(&self) -> impl Iterator<Item = &Game> {
         self.games.values()
     }
 
@@ -74,7 +83,7 @@ impl GameDb {
         games
             .into_iter()
             .try_for_each(|game| {
-                if let Some(game) = self.games.get(game.as_ref()) {
+                if let Some(game) = self.game(game.as_ref()) {
                     parts.extend(game.parts.values().cloned());
                     Ok(())
                 } else {
@@ -103,7 +112,7 @@ impl GameDb {
     }
 
     fn verify_game(&self, root: &Path, game_name: &str) -> Vec<VerifyFailure> {
-        if let Some(game) = self.games.get(game_name) {
+        if let Some(game) = self.game(game_name) {
             let mut results = game.parts.verify_failures(&root.join(game_name));
             results.extend(
                 game.devices
@@ -118,15 +127,13 @@ impl GameDb {
 
     pub fn list_results(&self, search: Option<&str>, simple: bool) -> Vec<GameRow> {
         if let Some(search) = search {
-            self.games
-                .values()
+            self.games_iter()
                 .filter(|g| !g.is_device)
                 .map(|g| g.report(simple))
                 .filter(|g| g.matches(search))
                 .collect()
         } else {
-            self.games
-                .values()
+            self.games_iter()
                 .filter(|g| !g.is_device)
                 .map(|g| g.report(simple))
                 .collect()
@@ -147,7 +154,7 @@ impl GameDb {
         GameDb::display_report(
             &games
                 .into_iter()
-                .filter_map(|g| self.games.get(g.as_ref()).map(|g| g.report(simple)))
+                .filter_map(|g| self.game(g.as_ref()).map(|g| g.report(simple)))
                 .collect::<Vec<GameRow>>(),
         )
     }
@@ -161,8 +168,7 @@ impl GameDb {
         let mut results: Vec<GameRow> = games
             .iter()
             .filter_map(|g| {
-                self.games
-                    .get(g)
+                self.game(g)
                     .filter(|g| !g.is_device)
                     .map(|g| g.report(simple))
             })
@@ -350,22 +356,26 @@ impl Game {
     }
 }
 
-fn read_game_dir<'s, I, F>(dir: I, files_on_disk: &DashMap<String, PathBuf>, failure: &mut F)
+fn read_game_dir<'s, I, S, F>(dir: I) -> (S, F)
 where
     I: Iterator<Item = std::io::Result<std::fs::DirEntry>>,
-    F: Extend<VerifyFailure<'s>>,
+    S: Default + ExtendOne<(String, PathBuf)>,
+    F: Default + ExtendOne<VerifyFailure<'s>>,
 {
+    let mut files_on_disk = S::default();
+    let mut failures = F::default();
+
     for entry in dir
         .filter_map(|e| e.ok())
         .filter(|e| e.file_type().map(|t| t.is_file()).unwrap_or(false))
     {
         match entry.file_name().into_string() {
-            Ok(name) => {
-                files_on_disk.insert(name, entry.path());
-            }
-            Err(_) => failure.extend(Some(VerifyFailure::extra(entry.path()))),
+            Ok(name) => files_on_disk.extend_one((name, entry.path())),
+            Err(_) => failures.extend_one(VerifyFailure::extra(entry.path())),
         }
     }
+
+    (files_on_disk, failures)
 }
 
 #[derive(Debug, Default, Serialize, Deserialize)]
@@ -442,8 +452,8 @@ impl GameParts {
         handle_failure: H,
     ) -> Result<(S, F), E>
     where
-        S: Default + Extend<VerifySuccess<'s>> + Send,
-        F: Default + Extend<VerifyFailure<'s>> + Send,
+        S: Default + ExtendOne<VerifySuccess<'s>> + Send,
+        F: Default + ExtendOne<VerifyFailure<'s>> + Send,
         I: Fn() + Send + Sync,
         H: Fn(VerifyFailure) -> Result<Result<(), VerifyFailure>, E> + Send + Sync,
         E: Send,
@@ -451,14 +461,9 @@ impl GameParts {
         use rayon::prelude::*;
         use std::sync::Mutex;
 
-        let mut failures = F::default();
-
-        // turn files on disk into a map, so extra files can be located
-        let files_on_disk: DashMap<String, PathBuf> = DashMap::new();
-
-        if let Ok(dir) = std::fs::read_dir(&game_root) {
-            read_game_dir(dir, &files_on_disk, &mut failures);
-        }
+        let (files_on_disk, failures): (DashMap<_, _>, F) = std::fs::read_dir(&game_root)
+            .map(read_game_dir)
+            .unwrap_or_default();
 
         let successes = Mutex::new(S::default());
         let failures = Mutex::new(failures);
@@ -467,15 +472,15 @@ impl GameParts {
         self.parts.par_iter().try_for_each(|(name, part)| {
             match files_on_disk.remove(name) {
                 Some((_, pathbuf)) => match part.verify(name, pathbuf) {
-                    Ok(success) => successes.lock().unwrap().extend(Some(success)),
+                    Ok(success) => successes.lock().unwrap().extend_one(success),
 
                     Err(failure) => match handle_failure(failure)? {
                         Ok(()) => successes
                             .lock()
                             .unwrap()
-                            .extend(Some(VerifySuccess { name, part })),
+                            .extend_one(VerifySuccess { name, part }),
 
-                        Err(failure) => failures.lock().unwrap().extend(Some(failure)),
+                        Err(failure) => failures.lock().unwrap().extend_one(failure),
                     },
                 },
 
@@ -488,9 +493,9 @@ impl GameParts {
                         Ok(()) => successes
                             .lock()
                             .unwrap()
-                            .extend(Some(VerifySuccess { name, part })),
+                            .extend_one(VerifySuccess { name, part }),
 
-                        Err(failure) => failures.lock().unwrap().extend(Some(failure)),
+                        Err(failure) => failures.lock().unwrap().extend_one(failure),
                     }
                 }
             }
@@ -503,7 +508,7 @@ impl GameParts {
         let mut failures = failures.into_inner().unwrap();
 
         // mark any leftover files on disk as extras
-        failures.extend(
+        failures.extend_many(
             files_on_disk
                 .into_iter()
                 .map(|(_, pb)| VerifyFailure::extra(pb)),
@@ -512,15 +517,6 @@ impl GameParts {
         Ok((successes.into_inner().unwrap(), failures))
     }
 
-    #[inline]
-    pub fn verify<'s, S, F>(&'s self, game_root: &Path) -> (S, F)
-    where
-        S: Default + Extend<VerifySuccess<'s>> + Send,
-        F: Default + Extend<VerifyFailure<'s>> + Send,
-    {
-        self.verify_with_progress(game_root, || {})
-    }
-
     pub fn verify_with_progress<'s, S, F, I>(
         &'s self,
         game_root: &Path,
@@ -528,8 +524,8 @@ impl GameParts {
     ) -> (S, F)
     where
         I: Fn() + Send + Sync,
-        S: Default + Extend<VerifySuccess<'s>> + Send,
-        F: Default + Extend<VerifyFailure<'s>> + Send,
+        S: Default + ExtendOne<VerifySuccess<'s>> + Send,
+        F: Default + ExtendOne<VerifyFailure<'s>> + Send,
     {
         self.process_parts(
             game_root,
@@ -540,8 +536,17 @@ impl GameParts {
     }
 
     #[inline]
-    pub fn verify_failures(&self, game_root: &Path) -> Vec<VerifyFailure> {
-        let (_, failures): (ExtendSink<VerifySuccess>, Vec<_>) = self.verify(game_root);
+    pub fn verify<'s, S, F>(&'s self, game_root: &Path) -> (S, F)
+    where
+        S: Default + ExtendOne<VerifySuccess<'s>> + Send,
+        F: Default + ExtendOne<VerifyFailure<'s>> + Send,
+    {
+        self.verify_with_progress(game_root, || {})
+    }
+
+    #[inline]
+    pub fn verify_failures<'s>(&'s self, game_root: &Path) -> Vec<VerifyFailure<'s>> {
+        let (_, failures): (ExtendSink<_>, _) = self.verify(game_root);
         failures
     }
 
@@ -554,8 +559,8 @@ impl GameParts {
         handle_failure: H,
     ) -> Result<(S, F), Error>
     where
-        S: Default + Extend<VerifySuccess<'s>> + Send,
-        F: Default + Extend<VerifyFailure<'s>> + Send,
+        S: Default + ExtendOne<VerifySuccess<'s>> + Send,
+        F: Default + ExtendOne<VerifyFailure<'s>> + Send,
         I: Fn() + Send + Sync,
         H: Fn(ExtractedPart<'_>) + Send + Sync + Copy,
     {
@@ -572,8 +577,8 @@ impl GameParts {
         handle_failure: H,
     ) -> Result<(S, F), Error>
     where
-        S: Default + Extend<VerifySuccess<'s>> + Send,
-        F: Default + Extend<VerifyFailure<'s>> + Send,
+        S: Default + ExtendOne<VerifySuccess<'s>> + Send,
+        F: Default + ExtendOne<VerifyFailure<'s>> + Send,
         H: Fn(ExtractedPart<'_>) + Send + Sync + Copy,
     {
         self.add_and_verify_with_progress(rom_sources, game_root, || {}, handle_failure)
@@ -589,9 +594,8 @@ impl GameParts {
     where
         H: Fn(ExtractedPart<'_>) + Send + Sync + Copy,
     {
-        let (_, failures): (ExtendSink<VerifySuccess<'s>>, Vec<_>) =
-            self.add_and_verify(rom_sources, game_root, handle_failure)?;
-        Ok(failures)
+        self.add_and_verify(rom_sources, game_root, handle_failure)
+           .map(|(_, failures): (ExtendSink<_>, _)| failures)
     }
 }
 
@@ -778,6 +782,42 @@ impl<'u> fmt::Display for ExtractedPart<'u> {
     }
 }
 
+// a simple polyfill until extend_one stabilizes in the Extend trait
+pub trait ExtendOne<I> {
+    fn extend_one(&mut self, item: I);
+
+    fn extend_many<T>(&mut self, iter: T)
+    where
+        T: IntoIterator<Item = I>,
+    {
+        for i in iter {
+            self.extend_one(i);
+        }
+    }
+}
+
+impl<I> ExtendOne<I> for Vec<I> {
+    #[inline]
+    fn extend_one(&mut self, item: I) {
+        self.push(item);
+    }
+
+    #[inline]
+    fn extend_many<T>(&mut self, iter: T)
+    where
+        T: IntoIterator<Item = I>,
+    {
+        self.extend(iter);
+    }
+}
+
+impl<K: Eq + std::hash::Hash, V> ExtendOne<(K, V)> for DashMap<K, V> {
+    #[inline]
+    fn extend_one(&mut self, (key, value): (K, V)) {
+        self.insert(key, value);
+    }
+}
+
 struct ExtendSink<I>(std::marker::PhantomData<I>);
 
 impl<I> Default for ExtendSink<I> {
@@ -787,9 +827,14 @@ impl<I> Default for ExtendSink<I> {
     }
 }
 
-impl<I> Extend<I> for ExtendSink<I> {
+impl<I> ExtendOne<I> for ExtendSink<I> {
     #[inline]
-    fn extend<T>(&mut self, _: T)
+    fn extend_one(&mut self, _: I) {
+        // do nothing
+    }
+
+    #[inline]
+    fn extend_many<T>(&mut self, _: T)
     where
         T: IntoIterator<Item = I>,
     {
@@ -797,6 +842,28 @@ impl<I> Extend<I> for ExtendSink<I> {
     }
 }
 
+pub struct ExtendExists<I> {
+    pub exists: bool,
+    phantom: std::marker::PhantomData<I>,
+}
+
+impl<I> Default for ExtendExists<I> {
+    #[inline]
+    fn default() -> Self {
+        Self {
+            exists: false,
+            phantom: std::marker::PhantomData,
+        }
+    }
+}
+
+impl<I> ExtendOne<I> for ExtendExists<I> {
+    #[inline]
+    fn extend_one(&mut self, _: I) {
+        self.exists = true;
+    }
+}
+
 #[derive(Clone, Hash, PartialEq, Eq)]
 pub struct FileId {
     pub dev: u64,
@@ -906,8 +973,10 @@ impl Part {
 
     #[inline]
     pub fn get_xattr(path: &Path) -> Option<Self> {
-        match xattr::get(path, CACHE_XATTR) {
-            Ok(Some(v)) => match v.split_first() {
+        xattr::get(path, CACHE_XATTR)
+            .ok()
+            .flatten()
+            .and_then(|v| match v.split_first() {
                 Some((b'r', sha1_hex)) => {
                     let mut sha1 = [0; 20];
                     hex::decode_to_slice(sha1_hex, &mut sha1)
@@ -921,9 +990,7 @@ impl Part {
                         .ok()
                 }
                 _ => None,
-            },
-            _ => None,
-        }
+            })
     }
 
     #[inline]
